/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.agent.model.entity.FinanceDO;
import com.zayac.admin.agent.model.entity.StatsDO;
import com.zayac.admin.agent.model.req.FinanceReq;
import com.zayac.admin.agent.model.req.FinanceSumReq;
import com.zayac.admin.agent.service.FinanceService;
import com.zayac.admin.agent.service.FinanceSumService;
import com.zayac.admin.agent.service.StatsService;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.req.AgentDataVisualListReq;
import com.zayac.admin.req.team.TeamFinanceReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.resp.AgentDataVisualList;
import com.zayac.admin.resp.Statics;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamAccountFinance;
import com.zayac.admin.resp.team.TeamFinancePagination;
import com.zayac.admin.service.AgentDataVisualListService;
import com.zayac.admin.service.CompletableFutureFinanceService;
import com.zayac.admin.service.TeamService;
import com.zayac.admin.service.TelegramMessageService;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.*;
import com.zayac.admin.utils.TableFormatter;
import lombok.RequiredArgsConstructor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import top.continew.starter.core.exception.BusinessException;

import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;

@Component
@RequiredArgsConstructor
public class DailyReport {
    private final UserService userService;
    private final TeamService teamService;
    private final RoleService roleService;
    private final UserRoleService userRoleService;
    private final AccountService accountService;
    private final TelegramMessageService telegramMessageService;
    private final AgentDataVisualListService agentDataVisualListService;
    private final StatsService statsService;
    private final CompletableFutureFinanceService completableFutureFinanceService;
    private final FinanceService financeService;
    private final FinanceSumService financeSumService;

    private static final String MINISTER_ROLE_CODE = "minister";

    @Scheduled(cron = "0 40 11,14,17,21 * * ?")
    public void teamAccountDailyReport() {
        LocalDateTime nowDateTime = LocalDateTime.now();
        LocalDate nowDate = LocalDate.now();
        sendDailyReport(nowDate, nowDate.atStartOfDay(), nowDateTime);
    }

    @Scheduled(cron = "0 15 0 * * ?")
    public void dailySummarize() {
        LocalDate yesterday = LocalDate.now().minusDays(1);
        sendDailyReport(yesterday, yesterday.atStartOfDay(), LocalDateTime.of(yesterday, LocalTime.MAX));
        saveData(yesterday, yesterday.atStartOfDay(), LocalDateTime.of(yesterday, LocalTime.MAX));
    }


    private void sendDailyReport(LocalDate reportDate, LocalDateTime startDateTime, LocalDateTime endDateTime) {
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());

        userIds.parallelStream().forEach(userId -> {
            UserDO ministerUser = userService.getById(userId);
            List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, ministerUser.getDeptId());
            if (ministerUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                List<String[]> rows = generateRows(ministerUser.getId(), startDateTime, endDateTime);
                String table = TableFormatter.formatTableAsHtml(rows);
                //telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getReportIds(), table);
                telegramMessageService
                        .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, table);
            }

            AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder()
                    .monthDate(reportDate)
                    .build();
            //获取部长对应下级所有用户
            deptUsers.parallelStream().forEach(deptUser -> {
                String message = getDeptUserMessage(deptUser, agentDataVisualListReq, reportDate);
                if (StrUtil.isNotBlank(message) && deptUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                    String botToken = StrUtil.isEmpty(deptUser.getBotToken()) ? ministerUser.getBotToken() : deptUser.getBotToken();
                    telegramMessageService.sendMessage(botToken, deptUser.getReportIds(), message);
                }
            });
        });
    }


    //获取需要发送给用户的信息
    private String getDeptUserMessage(UserDO deptUser,
                                      AgentDataVisualListReq agentDataVisualListReq,
                                      LocalDate reportDate) {
        StringBuilder message = new StringBuilder();
        // 主线注册新增不用报数,过滤
        List<AccountResp> currUserAccounts = accountService.getAccountsByUserId(deptUser.getId(), DisEnableStatusEnum.ENABLE)
                .stream()
                .filter(accountResp -> !accountResp.getIsTeam())
                .toList();

        currUserAccounts.forEach(currAccount -> {
            CompletableFuture<AgentDataVisualList> future = agentDataVisualListService
                    .getAgentDataVisualList(currAccount, agentDataVisualListReq);
            AgentDataVisualList agentData = future.join();
            Statics statics = agentData.getCurData()
                    .stream()
                    .filter(data -> data.getStaticsDate().equals(reportDate))
                    .findFirst()
                    .orElseThrow();
            message.append(currAccount.getUsername())
                    .append("\n")
                    .append("注册: ")
                    .append(statics.getIsNew())
                    .append("\n")
                    .append("新增: ")
                    .append(statics.getFirstCount())
                    .append("\n")
                    .append("日活: ")
                    .append(statics.getCountBets())
                    .append("\n\n");
        });

        return message.toString();
    }


    private void saveData(LocalDate reportDate, LocalDateTime startDateTime, LocalDateTime endDateTime) {
        // 获取传入年月
        YearMonth inputYearMonth = YearMonth.from(reportDate);

        // 获取当前年月和上个月的年月
        YearMonth currentYearMonth = YearMonth.now();
        YearMonth previousYearMonth = currentYearMonth.minusMonths(1);

        if (inputYearMonth.equals(currentYearMonth) || inputYearMonth.equals(previousYearMonth)) {
            RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
            List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());

            userIds.forEach(userId -> {
                UserDO ministerUser = userService.getById(userId);
                List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, ministerUser.getDeptId());
                TeamFinanceReq teamFinanceReq = TeamFinanceReq.builder()
                        .pageNum(1)
                        .pageSize(999)
                        .commissionDate(reportDate)
                        .build();
                List<AccountResp> ministerUserAccounts = accountService.getAccountsByUserId(ministerUser.getId(), DisEnableStatusEnum.ENABLE)
                        .stream()
                        .filter(AccountResp::getIsTeam)
                        .toList();
                ministerUserAccounts.parallelStream().forEach(accountResp -> {
                    TeamFinancePagination<List<TeamAccountFinance>> financePagination = completableFutureFinanceService.getTeamFinance(accountResp, teamFinanceReq).join();
                    List<FinanceDO> financeReqList = financePagination.getList().stream().map(finance -> {
                        FinanceDO financeDO = new FinanceDO();
                        BeanUtil.copyProperties(finance, financeDO);
                        return financeDO;
                    }).toList();
                    financeService.addAll(financeReqList);
                    FinanceSumReq financeSumReq = new FinanceSumReq();
                    BeanUtil.copyProperties(financePagination.getTotalSumVo(),financeSumReq);
                    financeSumService.add(financeSumReq);
                });
                AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder()
                        .monthDate(reportDate)
                        .build();
                deptUsers.parallelStream().forEach(deptUser -> {
                    List<AccountResp> currUserAccounts = accountService.getAccountsByUserId(deptUser.getId(), DisEnableStatusEnum.ENABLE)
                            .stream()
                            .filter(accountResp -> !accountResp.getIsTeam())
                            .toList();
                    List<StatsDO> list = currUserAccounts.parallelStream().map(currAccount -> {
                                CompletableFuture<AgentDataVisualList> future = agentDataVisualListService
                                        .getAgentDataVisualList(currAccount, agentDataVisualListReq);
                                AgentDataVisualList agentData = future.join();
                                Statics statics = agentData.getCurData()
                                        .stream()
                                        .filter(data -> data.getStaticsDate().equals(reportDate))
                                        .findFirst()
                                        .orElseThrow();
                                StatsDO statsDO = new StatsDO();
                                BeanUtil.copyProperties(statics, statsDO);
                                return statsDO;
                            }
                    ).toList();
                    statsService.addAll(list);
                });

            });
        } else {
            throw new BusinessException("只允许查询当月以及上个月的数据");
        }
    }


    public static String getPercent(int x, int y) {
        double d1 = x * 1.0;
        double d2 = y * 1.0;
        NumberFormat percentInstance = NumberFormat.getPercentInstance();
        percentInstance.setMinimumFractionDigits(2);
        return percentInstance.format(d1 / d2);
    }

    private List<String[]> generateRows(Long userId, LocalDateTime startDateTime, LocalDateTime endDateTime) {
        List<String[]> rows = new ArrayList<>();
        rows.add(new String[]{"平台", "注册", "新增", "转化率"});
        rows.add(new String[]{"----", "----", "----", "----"});

        List<AccountResp> accounts = accountService.getAccountsByUserId(userId, DisEnableStatusEnum.ENABLE)
                .stream()
                .filter(AccountResp::getIsTeam)
                .toList();
        int[] totals = {0, 0};
        TeamInfoReq teamInfoReq = TeamInfoReq.builder().startDate(startDateTime).endDate(endDateTime).build();
        accounts.forEach(accountResp -> {
            Team team = teamService.getLatestTeamInfoAsync(accountResp, teamInfoReq).join();
            int totalNewMember = team.getList().stream().mapToInt(TeamAccount::getSubMemberNum).sum();
            int totalNewFirstDeposit = team.getList().stream().mapToInt(TeamAccount::getFirstDepositNum).sum();
            totals[0] += totalNewMember;
            totals[1] += totalNewFirstDeposit;
            String percent = getPercent(totalNewFirstDeposit, totalNewMember);
            rows.add(new String[]{accountResp.getPlatformName(), String.valueOf(totalNewMember), String.valueOf(totalNewFirstDeposit), percent});
        });

        rows.add(new String[]{"总注册", String.valueOf(totals[0]), String.valueOf(totals[1]), getPercent(totals[1], totals[0])});
        return rows;
    }
}
